@@ -35,7 +35,10 @@ class TripleOAction(base.Action):
 
         kwargs = {
             'preauthurl': obj_ep.url % {'tenant_id': ctx.project_id},
-            'preauthtoken': ctx.auth_token
+            'preauthtoken': ctx.auth_token,
+            'retries': 10,
+            'starting_backoff': 3,
+            'max_backoff': 120
         }
 
         return swift_client.Connection(**kwargs)
@@ -53,7 +56,7 @@ class TripleOAction(base.Action):
             ironic_endpoint.url,
             token=ctx.auth_token,
             region_name=ironic_endpoint.region,
-            os_ironic_api_version='1.11',
+            os_ironic_api_version='1.15',
             # FIXME(lucasagomes):Paramtetize max_retries and
             # max_interval. At the moment since we are dealing with
             # a critical bug (#1612622) let's just hardcode the times
