@@ -120,9 +120,8 @@ class UpdateParametersAction(base.TripleOAction):
 class UpdateRoleParametersAction(UpdateParametersAction):
     """Updates role related parameters in Mistral Environment ."""
 
-    def __init__(self, role, container=constants.DEFAULT_CONTAINER_NAME):
-        super(UpdateRoleParametersAction, self).__init__(parameters=None,
-                                                         container=container)
+    def __init__(self, role):
+        super(UpdateRoleParametersAction, self).__init__(parameters=None)
         self.role = role
 
     def run(self):
