@@ -18,6 +18,7 @@ import yaml
 
 from heatclient import exc as heatexceptions
 from mistral.workflow import utils as mistral_workflow_utils
+from mistralclient.api import base as mistralclient_base
 from swiftclient import exceptions as swiftexceptions
 
 from tripleo_common.actions import base
@@ -75,6 +76,18 @@ class CreatePlanAction(base.TripleOAction):
         }
         env_vars = {}
         error_text = None
+
+        # Check to see if an environment with that name already exists
+        try:
+            self._get_workflow_client().environments.get(self.container)
+        except mistralclient_base.APIException:
+            # The environment doesn't exist, as expected. Proceed.
+            pass
+        else:
+            message = ("Unable to create plan. The Mistral environment "
+                       "already exists")
+            return mistral_workflow_utils.Result(None, message)
+
         try:
             # parses capabilities to get root_template, root_environment
             mapfile = yaml.safe_load(
@@ -103,6 +116,60 @@ class CreatePlanAction(base.TripleOAction):
             return mistral_workflow_utils.Result(error=error_text)
 
 
+class UpdatePlanAction(base.TripleOAction):
+    """Update a plan
+
+    Given a container, update the Mistral environment with the same name,
+    parses the capabilities map file and sets the initial plan template
+    and environment files if they have changed since the plan was originally
+    created.
+    """
+
+    def __init__(self, container):
+        super(UpdatePlanAction, self).__init__()
+        self.container = container
+
+    def run(self):
+        swift = self._get_object_client()
+        mistral = self._get_workflow_client()
+
+        error_text = None
+
+        try:
+            mapobject = swift.get_object(self.container,
+                                         'capabilities-map.yaml')[1]
+            mapfile = yaml.safe_load(mapobject)
+
+            mistral_env = mistral.environments.get(self.container)
+
+            # We always want the root template to match whatever is in the
+            # capabilities map, so update that regardless.
+            mistral_env.variables['root_template'] = mapfile['root_template']
+
+            # Check to see if the root environment is already listed, if it
+            # isn't - add it.
+            # TODO(d0ugal): Users could get into a situation where they have
+            #               an old root environment still added and they
+            #               then have two after the new one is added.
+            root_env = {'path': mapfile['root_environment']}
+            if root_env not in mistral_env.variables['environments']:
+                mistral_env.variables['environments'].insert(0, root_env)
+
+            mistral.environments.update(
+                name=self.container,
+                variables=mistral_env.variables,
+            )
+        except yaml.YAMLError as yaml_err:
+            error_text = "Error parsing the yaml file: %s" % yaml_err
+        except swiftexceptions.ClientException as obj_err:
+            error_text = "File missing from container: %s" % obj_err
+        except KeyError as key_err:
+            error_text = ("capabilities-map.yaml missing key: "
+                          "%s" % key_err)
+        if error_text:
+            return mistral_workflow_utils.Result(error=error_text)
+
+
 class ListPlansAction(base.TripleOAction):
     """Lists deployment plans
 
@@ -185,3 +252,39 @@ class DeletePlanAction(base.TripleOAction):
 
         if error_text:
             return mistral_workflow_utils.Result(error=error_text)
+
+
+class ListRolesAction(base.TripleOAction):
+    """Returns a deployment plan's roles
+
+    Parses overcloud.yaml and returns the Heat resources where
+    type = OS::Heat::ResourceGroup
+
+    :param container: name of the Swift container / plan name
+    :return: list of roles in the container's deployment plan
+    """
+
+    def __init__(self, container=constants.DEFAULT_CONTAINER_NAME):
+        super(ListRolesAction, self).__init__()
+        self.container = container
+
+    def run(self):
+        try:
+            mc = self._get_workflow_client()
+            mistral_env = mc.environments.get(self.container)
+            template_name = mistral_env.variables['template']
+
+            oc = self._get_object_client()
+            resources = yaml.safe_load(
+                oc.get_object(self.container, template_name)[1])['resources']
+        except Exception as mistral_err:
+            err_msg = ("Error retrieving deployment plan: %s"
+                       % mistral_err)
+            LOG.exception(err_msg)
+            return mistral_workflow_utils.Result(None, err_msg)
+
+        roles = []
+        for resource, details in resources.items():
+            if details['type'] == constants.RESOURCE_GROUP_TYPE:
+                roles.append(resource)
+        return roles
