@@ -157,7 +157,7 @@ class iBootDriverInfo(PrefixedDriverInfo):
 
 DRIVER_INFO = {
     # production drivers
-    '.*_ipmi(tool|native)': PrefixedDriverInfo('ipmi', has_port=True),
+    '.*_ipmi(tool|native)': PrefixedDriverInfo('ipmi'),
     '.*_drac': PrefixedDriverInfo('drac', address_field='host'),
     '.*_ilo': PrefixedDriverInfo('ilo'),
     '.*_ucs': PrefixedDriverInfo(
